#ifndef 	DS1307_C_
#define		DS1307_C_

#include  	<avr/io.h>          
#include  	<avr/interrupt.h>	    
#include  	<util/twi.h>
#include  	<stdio.h>
#include  	<string.h>

#include 	"serial.h"
#include 	"printf.h"

#include	"I2C.h"
#include 	"ds1307.h"
#include 	"mydefs.h"                 


extern uint8_t	flag;
uint8_t 		rtc_register[10];

//-----------------------------------------------------------------------------//
uint8_t BinToBcd(uint8_t val)
{
	return ((val / 10) *16 + val % 10);
}
//-----------------------------------------------------------------------------//
uint8_t BcdToBin(uint8_t val)
{
	return (val >> 4) * 10 + (val & 0x0f); 
}
//-----------------------------------------------------------------------------------//
void RTC_setStartAddress(void)
{
   uint8_t errorStatus;
   
   errorStatus = i2c_start();
   if(errorStatus == 1)
   {
      	i2c_stop();
	 return;
   } 
   
   errorStatus = i2c_sendAddress(DS1307_W);
   
   if(errorStatus == 1)
   {
    
	 i2c_stop();
	 return;
   } 
   
   errorStatus = i2c_sendData(0x00);
   if(errorStatus == 1)
   {
     
	 i2c_stop();
	 return;
   } 

   i2c_stop();
}
//-----------------------------------------------------------------------------------//
uint8_t DS1307_Read (void)
{

	uint8_t errorStatus, i, data;
  
	errorStatus = i2c_start();
	
	if(errorStatus == 1)
	{
		i2c_stop();
		return 0;
	} 
   
	errorStatus = i2c_sendAddress(DS1307_W);
   
	if(errorStatus == 1)
	{
		i2c_stop();
		return 0;
	} 
   
	errorStatus = i2c_sendData(0x00);
	if(errorStatus == 1)
	{
		i2c_stop();
		return 0;
	} 

    errorStatus = i2c_repeatStart();
	if(errorStatus == 1)
	{
      	i2c_stop();
		return 0;
	} 
   
    errorStatus = i2c_sendAddress(DS1307_R);
   
	if(errorStatus == 1)
	{
		i2c_stop();
		return 0;
	} 
 
    for(i=0;i<7;i++)
	{
		if(i == 6)  	 //no Acknowledge after receiving the last byte
			data = i2c_receiveData_NACK();
		else
			data = i2c_receiveData_ACK();
		  
		if(data == ERROR_CODE)
		{
       		i2c_stop();
	   		return 0;
		}
	  
		rtc_register[i] = data;
	}
	i2c_stop();
	return 1;
}
//-----------------------------------------------------------------------------------//
//-----------------------------------------------------------------------------//
uint8_t DS1307_Write (void)
{
	uint8_t errorStatus, i;
  
	errorStatus = i2c_start();
	if(errorStatus == 1)
	{
		i2c_stop();
		return 1 ;
	} 
   
	errorStatus = i2c_sendAddress(DS1307_W);
   
	if(errorStatus == 1)
	{
		i2c_stop();
		return 1;
	} 
   
	errorStatus = i2c_sendData(0x00);
	if(errorStatus == 1)
	{
		i2c_stop();
		return 1;
	} 

    for(i=0;i < 8;i++)
    {
		errorStatus = i2c_sendData(rtc_register[i]);  
		if(errorStatus == 1)
		{
       		i2c_stop();
	   		return 1;
		}
    }
	
	i2c_stop();
	return(0);
}
//-----------------------------------------------------------------------------//
uint8_t DS1307_Write_RTC (uint8_t address, uint8_t data)
{
	uint8_t errorStatus;
  
	errorStatus = i2c_start();
	if(errorStatus == 1)
	{
		i2c_stop();
		return  1;
	} 
   
	errorStatus = i2c_sendAddress(DS1307_W);
   
	if(errorStatus == 1)
	{
		i2c_stop();
		return 1;
	} 
   
	errorStatus = i2c_sendData(address);
	if(errorStatus == 1)
	{
		i2c_stop();
		return 1;
	} 
	errorStatus = i2c_sendData(data);  
	if(errorStatus == 1)
	{
       		i2c_stop();
	   		return 1 ;
    }
	
	i2c_stop();
	return 0;
}
//-----------------------------------------------------------------------------//
void Read_RTC (void) 
{	
	DS1307_Read ();
	
	RTC.sec   = BcdToBin (rtc_register[0]);
	RTC.min   = BcdToBin (rtc_register[1]); 
	RTC.hour  = BcdToBin (rtc_register[2]);
		
	RTC.day	  = BcdToBin (rtc_register[3]);
	
	RTC.date  = BcdToBin (rtc_register[4]);
	RTC.month = BcdToBin (rtc_register[5]); 
	RTC.year  = BcdToBin (rtc_register[6]); //last read just to set ACK
}
//-----------------------------------------------------------------------------//
void update_RTC (void) 
{
	rtc_register[0] = BinToBcd   	(RTC.sec);
	rtc_register[1] = BinToBcd   	(RTC.min);
	rtc_register[2] = BinToBcd   	(RTC.hour);
	
	rtc_register[3] = BinToBcd   	(RTC.day);
	
	rtc_register[4] = BinToBcd   	(RTC.date);
	rtc_register[5] = BinToBcd   	(RTC.month);
	rtc_register[6] = BinToBcd   	(RTC.year);
	
	rtc_register[7]  = 0x90;
	
	DS1307_Write ();
}
//-----------------------------------------------------------------------------//
//Disable RTC interrupt
void disable_rtc (void)
{
	RTC_INTREG &= ~_BV(INT0);
}
//-----------------------------------------------------------------------------//
//Enable RTC interrupt
void enable_rtc (void)
{
	RTC_INTREG |= _BV(INT0);
}
//------------------------------------------------------------------------//
char febuary (uint32_t lvalue)
{
//year % 4 == TRUE date = 29
	char 		nret = 1;
	uint16_t	value;
/* 
//jnut debug
			RTC.date =  30;
			RTC.month = 2;
			RTC.year  = 12;
*/		
		RTC.date  = (uint8_t)( lvalue / 10000);
		RTC.month = (uint8_t)((lvalue / 100)%100);
		RTC.year  = (uint8_t)( lvalue % 100);
		
		if ((((RTC.date  < 32)  && (RTC.date  > 0)) && 
			 ((RTC.month < 13) && (RTC.month > 0)) &&
			  (RTC.year  < 60)))
		{
			value = 2000 + RTC.year;
			if ((value%4) && (RTC.month == 2))
			{
				printf ("\r\nFEB %02d",RTC.date);
				if (RTC.date > 28)	nret = 0;
			}
			else
			   nret = 1;
		}
		else
			nret = 0;
	return nret;
}
//-----------------------------------------------------------------------------------//
char check_time (uint32_t lvalue)
{
	char nret = 0;
				
		RTC.hour  	= (uint8_t)( lvalue / 3600);
		RTC.min 	= (uint8_t)((lvalue / 60)%60);
		RTC.sec  	= (uint8_t)( lvalue % 60);
			
		if ((RTC.hour < 25) && (RTC.min < 60) && (RTC.sec < 60))
		{
			nret = 1;
		}	
	return nret;
}
//-----------------------------------------------------------------------------//
int leap (int year)
{
	return year*365 + (year/4) - (year/100) + (year/400);
}
//-----------------------------------------------------------------------------//
int zeller (int year, int month, int day)
{	
	year += ((month+9)/12);// OLD - 1;
	month = (month+9) % 12;
	
	return leap (year) + month*30 + ((6*month+5)/10) + day + 1;
}
//-----------------------------------------------------------------------------//  
uint8_t calculate_day (int year, int month, int day)
{
  return (zeller (year, month, day) % 7) + 1; 	//  day between 1- 7
												//  1 = Sunday
												//  7 = Saturday
}
//-----------------------------------------------------------------------------//
void check_DS1307 (char update)
{

	Read_RTC ();
	
	if ((RTC.sec == RTC_ERROR) || update)
	{
/*
J.nut Default 01/01/2014 Time 00:00:00 
*/	
		RTC.sec = RTC.min = RTC.hour = 0;
		RTC.date  = 01;
		RTC.month = 01;
		RTC.year  = 14;
			RTC.day   = calculate_day(RTC.year, RTC.month, RTC.date);
	}		
	update_RTC ();
}
//-----------------------------------------------------------------------------//
void new_update (void)
{
	RTC.day   = calculate_day(RTC.year, RTC.month, RTC.date);
	update_RTC ();
}
//-----------------------------------------------------------------------------//
uint32_t display_realtime (void)
{
	//char   	 lcdbuf[20];
	uint32_t tValue = 0;
		
		Read_RTC ();
		
		//memset (lcdbuf, 0, sizeof lcdbuf);
		//sprintf(lcdbuf,"%02d:%02d:%02d",RTC.hour,RTC.min,RTC.sec);
//		//printf ("\r\n%s",lcdbuf);
		//lcd_gotoxy (LINE1,1);
		//lcd_printf (lcdbuf);
	
					
	tValue =    ((uint32_t)(RTC.hour) * HOUR 	+ 
				 (uint16_t)(RTC.min)  * MIN 	+  RTC.sec); 
	return  tValue;
};

#endif  // DS1307_C_

