#ifndef 	I2C_C_
#define		I2C_C_

#include  	<avr/io.h>          
#include  	<avr/interrupt.h>	    
#include  	<util/twi.h>
#include  	<stdio.h>
#include  	<string.h>

#include 	"serial.h"
#include 	"printf.h"

#include 	"i2c.h"
#include 	"mydefs.h"  

//-----------------------------------------------------------------------------------//
char i2c_start (void)
{
 
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN); 		//Send START condition
	    while (!(TWCR & (1<<TWINT)));   			//Wait for TWINT flag set. This indicates that the
													//START condition has been transmitted
    if ((TWSR & 0xF8) == START)					//Check value of TWI Status Register
	   return(0);
	else		 	  	 				//if problem, transmit the code to PC  to know what's the problem
	   return(1);
  
}
//-----------------------------------------------------------------------------------//
char i2c_repeatStart (void)
{
 
    TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);  //Send START condition
    while (!(TWCR & (1<<TWINT)));   		 //Wait for TWINT flag set. This indicates that the
		  									 //START condition has been transmitted
    if ((TWSR & 0xF8) == REPEAT_START)		 //Check value of TWI Status Register
 	   return(0);
	else
	   return(1);
}
//-----------------------------------------------------------------------------------//
char i2c_sendAddress (uint8_t address)
{
   unsigned char STATUS;
   
   if((address & 0x01) == 0) 
     STATUS = MT_SLA_ACK;
   else
     STATUS = MR_SLA_ACK; 
   
   TWDR = address; 
   TWCR = (1<<TWINT)|(1<<TWEN);	   	   //Load SLA_W into TWDR Register. Clear TWINT bit
   		  			 				   //in TWCR to start transmission of address
   while (!(TWCR & (1<<TWINT)));	   //Wait for TWINT flag set. This indicates that the
   		 		   					   //SLA+W has been transmitted, and
									   //ACK/NACK has been received.
   if ((TWSR & 0xF8) == STATUS)	   	   //Check value of TWI Status Register
   	  return(0);
   else 
      return(1);
}
//-----------------------------------------------------------------------------------//
char i2c_sendData (uint8_t data)
{
   TWDR = data; 
   TWCR = (1<<TWINT) |(1<<TWEN);	   //Load SLA_W into TWDR Register. Clear TWINT bit
   		  			 				   //in TWCR to start transmission of data
   while (!(TWCR & (1<<TWINT)));	   //Wait for TWINT flag set. This indicates that the
   		 		   					   //data has been transmitted, and
									   //ACK/NACK has been received.
   if ((TWSR & 0xF8) != MT_DATA_ACK)   //Check value of TWI Status Register
   	  return(1);
   else
      return(0);
}
//-----------------------------------------------------------------------------------//
uint8_t i2c_receiveData_ACK (void)
{
  unsigned char data;
  
  TWCR = (1<<TWEA)|(1<<TWINT)|(1<<TWEN);
  
  while (!(TWCR & (1<<TWINT)));	   	   //Wait for TWINT flag set. This indicates that the
   		 		   					   //data has been received
  if ((TWSR & 0xF8) != MR_DATA_ACK)    //Check value of TWI Status Register
   	  return(ERROR_CODE);
  
  data = TWDR;
  return(data);
}
//-----------------------------------------------------------------------------------//
uint8_t i2c_receiveData_NACK(void)
{
  unsigned char data;
  
  TWCR = (1<<TWINT)|(1<<TWEN);
  
  while (!(TWCR & (1<<TWINT)));	   	   //Wait for TWINT flag set. This indicates that the
   		 		   					   //data has been received
  if ((TWSR & 0xF8) != MR_DATA_NACK)   //Check value of TWI Status Register
   	  return(ERROR_CODE);
  
  data = TWDR;
  return(data);	 
}
//-----------------------------------------------------------------------------------//
void i2c_stop (void)
{
	TWCR =  (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);	  //Transmit STOP condition
}
//-----------------------------------------------------------------------------------//
void twi_init (void) 
{
    /*
    SCL Clock frequency
    SCLf=FCPU/(16+2*TWBR*(4^TWPS))
    */
    TWBR = 2;
    TWSR |=	((1<<TWPS1)|(1<<TWPS0));

    /*
    Enable the TWI Module
    */
    TWCR|=	(1<<TWEN);
/*
 TWCR= 0x00; //disable twi
 TWBR= 0x12; //set bit rate
 TWSR= 0x01;
 TWAR= 0x00; //set slave address
 TWCR= 0x44; //enable twi


*/	
}
//-----------------------------------------------------------------------------------//
//-----------------------------------------------------------------------------------//
//-----------------------------------------------------------------------------------//
//-----------------------------------------------------------------------------------//
//-----------------------------------------------------------------------------------//
#endif

