#include 	<avr/io.h>
#include 	<avr/interrupt.h>
#include 	<avr/pgmspace.h>
#include 	<stdio.h>
#include 	<stdarg.h>
#include 	<string.h>
#include 	<stdlib.h>
#include	<math.h>

#include 	"mydefs.h"

#include 	"serial.h"
#include 	"printf.h"
//---------------------------------------------------------------------------------//
#include 	"util.h"
#include 	"i2c.h"
#include 	"ds1307.h"
#include 	"adc.h"
#include 	"p10.h"
#include 	"sign.h"
//---------------------------------------------------------------------------------//

uint8_t 	getByte,
			interrupt;
//---------------------------------------------------------------------------------//
char 		buff[30];
uint16_t 	flag;
char 		mode;


#define 	CLOCK		0
#define 	DATE		1
#define 	NTC			2

//---------------------------------------------------------------------------------//
#define 	KEY_IN			PINA
#define 	KEY_SETUP		_BV(SETUP)
#define 	KEY_ENT			_BV(ENTER)
#define 	KEY_ESC			_BV(ESC)
#define 	KEY_HP			_BV(HPLUS)
#define 	KEY_MP			_BV(MPLUS)
#define 	KEY_SP			_BV(SPLUS)
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
void init_port (void)
{
	LED_DDR |= 0xFC;
	PORTC    = 0xff;	//all off
	
	BUZ_DDR |= _BV(BUZ);
	
}
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
ISR(INT0_vect)
{
	flag 	|= DS1307_INT;		//Set Flag interrupt buffer after INT0 interrupt
}
//---------------------------------------------------------------------------------//	
void init_rtc (void)
{
#ifdef	RTC_1307
	/* set the I2C bit rate generator to 100 kb/s */
	twi_init ();
	//Enable INT 0 interrupt
	
	//  write logical 1 to ISC bit0 & bit1 
	MCUCR  = (1 << ISC01) | (0 << ISC00); //Falling egde interrupt
	
	check_DS1307 (0);
	
	//printf ("\r\ninit rtc");
	//INT0 interrupt
#endif
}
//---------------------------------------------------------------------------------//
void init_ntc (void)
{
	
	DDRA   = 0x00;			//
	PORTA |= 0xFE;			//Bit 1..7 Switch
							//Bit 0 NTC
							
	init_adc ();						
}
//---------------------------------------------------------------------------------//
void Read_Clock (int diff)
{
	
		Read_RTC ();
		
		memset  (buff, 0, sizeof buff);
		sprintf (buff,"%02d:%02d:%02d",		RTC.hour,
											RTC.min,
											RTC.sec + diff);
}
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
void Read_Date (void)
{
	
		Read_RTC ();
		
		memset  (buff, 0, sizeof buff);
		sprintf (buff,"%02d/%02d/%02d",		RTC.date,
											RTC.month,
											RTC.year);
}
//---------------------------------------------------------------------------------//
ISR (TIMER0_OVF_vect)
{
	TIMSK &= ~_BV(TOIE0);	//disable interrupt
	
		show (display[0]);
	
	TIMSK |= _BV(TOIE0); // enable interrupt
			
}
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//	
void disable_timmer (void)
{
    TIMSK &= ~_BV(TOIE0); 	// disable overflow interrupt
}
//---------------------------------------------------------------------------------//
void enable_timmer (void)
{
    TIMSK |= (1 << TOIE0);	// enable overflow interrupt
}
//---------------------------------------------------------------------------------//
void init_timmer (void)
{
// set up timer with prescaler = 256
    TCCR0 |= (1 << CS02);
  
    // initialize counter
    TCNT0 = 1;
  
    // enable overflow interrupt
    TIMSK |= (1 << TOIE0); 
}
//---------------------------------------------------------------------------------//			
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//			
//---------------------------------------------------------------------------------//
void show_message (char *str)
{
	memset (buff, 0, sizeof buff);
	sprintf (buff,"%s",str);
	Print_Font (0, buff);
		still (50);
}
//---------------------------------------------------------------------------------//
void show_mode (char mode)
{
		memset (buff, 0, sizeof buff);
		if (!mode)	sprintf (buff, "Set Date");
		else        sprintf (buff, "Set Time");
		Print_Font (0, buff);
}
//---------------------------------------------------------------------------------//
void update_screen (char mode)
{
	if (mode == SETDATE)	sprintf (buff, "%02d/%02d/%02d"	,RTC.date
														,RTC.month
														,RTC.year);
	else
	{
		if (mode == SETTIME) sprintf (buff, "%02d:%02d:%02d"	,RTC.hour
															,RTC.min
															,RTC.sec);
	
	}	
	Print_Font (0, buff);												
}
//---------------------------------------------------------------------------------//			
void set_date (void)
{
	char nexit = 1;
	uint32_t  wait_key = 0;
	
		memset  (buff, 0, sizeof buff);
		sprintf (buff, "New Date");
		Print_Font (0, buff);
		still (30);
		sprintf (buff, "%02d/%02d/%02d"	,RTC.date
										,RTC.month
										,RTC.year);
		Print_Font (0, buff);								
		
		while (nexit)
		{
			if (wait_key++ > 5000)
			{
				if (!(KEY_IN & KEY_ENT))
				{
					beep (LONG);
					show_message ("Update..");
					update_RTC ();
					nexit = 0;
				}
				else
				{
					if (!(KEY_IN & KEY_ESC))
					{
						beep (LONG);
						show_message ("Cancel..");
						nexit = 0;
					}	
				}
				
				
				if (!(KEY_IN & KEY_HP))
				{
					if (RTC.date++ >= 31) RTC.date = 1;
					beep (SHORT);
					update_screen (SETDATE);
				}
				else
				{
					if (!(KEY_IN & KEY_MP))
					{
						if (RTC.month++ >= 12) RTC.month = 1;
						beep (SHORT);
						update_screen (SETDATE);
					}
					else
					{
						if (!(KEY_IN & KEY_SP))
						{
							
							if (RTC.year++ >= 25) RTC.year = 14;
							beep (SHORT);
							update_screen (SETDATE);
						}
					}
				}
				wait_key = 0;
			}
		}
}
//---------------------------------------------------------------------------------//
void set_time (void)
{
	char nexit = 1;
	uint32_t  wait_key = 0;
	
		memset  (buff, 0, sizeof buff);
		sprintf (buff, "New Time");
		Print_Font (0, buff);
		still (30);
		sprintf (buff, "%02d:%02d:%02d"	,RTC.hour
										,RTC.min
										,RTC.sec);
		Print_Font (0, buff);
		while (nexit)
		{
			if (wait_key++ > 5000)
			{
				if (!(KEY_IN & KEY_ENT))
				{
					beep (LONG);
					show_message ("Update..");
					update_RTC ();
					nexit = 0;
				}
				else
				{
					if (!(KEY_IN & KEY_ESC))
					{
						beep (LONG);
						show_message ("Cancel..");
						nexit = 0;
					}	
				}
				
				
				if (!(KEY_IN & KEY_HP))
				{
					if (RTC.hour++ >= 23) RTC.hour = 0;
					beep (SHORT);
					update_screen (SETTIME);
				}
				else
				{
					if (!(KEY_IN & KEY_MP))
					{
						if (RTC.min++ >= 59) RTC.min = 0;
						beep (SHORT);
						update_screen (SETTIME);
					}
					else
					{
						if (!(KEY_IN & KEY_SP))
						{
							
							if (RTC.sec++ >= 59) RTC.sec = 0;
							beep (SHORT);
							update_screen (SETTIME);
						}
					}
				}
				wait_key = 0;
			}
		}
}
//---------------------------------------------------------------------------------//
void setup (char mode)
{
	Read_RTC ();
	
	if (mode == 0) set_date ();
	else if (mode == 1) set_time ();
}
//---------------------------------------------------------------------------------//
void menu (void)
{
	int mode = 0;
	char nexit = 1;
	
		memset (buff, 0, sizeof buff);
		show_mode (mode);
		
		while (nexit)
		{
			if (!(KEY_IN & KEY_SETUP))
			{
				mode += 1;
				if (mode > 1) mode = 0;
				show_mode (mode);
				beep (SHORT);
				while (!(KEY_IN & KEY_SETUP));
				
			}
			else
			{
				if (!(KEY_IN &  KEY_ENT))
				{
					beep (LONG);
					setup (mode);
					while (!(KEY_IN & KEY_ENT));
					nexit = 0;
				
				}
				else
				{
					if (!(KEY_IN & KEY_ESC))
					{
						beep (SHORT);
						while (!(KEY_IN & KEY_ESC));
						show_message ("Cancel..");
						nexit = 0;
						
					}
				}
			}
		}
					
		sprintf (buff, "%02d:%02d:%02d"	,RTC.hour
										,RTC.min
										,RTC.sec);
		Print_Font (1, buff);
		Print_Font (0, buff);
		
			
}			
//---------------------------------------------------------------------------------//
int main(void)
{
	double 	tmperature;
	
	int 	shift,
	digit_shift;
	
	ser_init();
	
	
	sei ();
	
	interrupt = 0;
	
	init_port ();
	
	init_rtc ();
	
	init_ntc ();
	
	beep (SHORT);
	
	printf ("\r\nReady....jnut");
	
	Read_Clock (0);
	
	Print_Font (0,buff);

	//RTC.sec = RTC.min = 25; 
	//RTC.hour = 16;
	//update_RTC ();	
	
	
	RTC.date = 05;
	RTC.month = 12; 
	RTC.year = 14;
	//update_RTC ();	

	
	mode = DATE;
	
	enable_rtc ();
	
	init_timmer ();
	
	enable_timmer ();

	
	while (1)
	{
	
		
		
		if (!(KEY_IN & KEY_SETUP))
		{	
			disable_rtc ();
			beep (SHORT);
			while(!(KEY_IN & KEY_SETUP));
			menu ();
			
			flag = 0;
			
			enable_rtc ();
			
		}
		
		if (flag & DS1307_INT)
		{
		
			switch (mode)
			{
				//-------------------------------------//
				case CLOCK :	Read_Clock (0);
								
				
								Print_Font (1,buff);	
								
								shift = calculate_shift (RTC.hour,RTC.min,RTC.sec);
								arrow_down (shift + digit_shift,1);
								digit_shift = 0;
								if (RTC.sec == 5)
								{
									keep_left (2);
									mode = DATE;
								}
								break;
				//-------------------------------------//
				case DATE :		clear_buff (0);
								Read_Date ();
								Print_Font (1,buff);	
								rain_down_new (2);
								//mode = NTC;
								//break;
								//shift_right (2);
								//break;
				//-------------------------------------//
				case  NTC :		tmperature = read_Temp ();
								memset (buff, 0, sizeof buff);
								sprintf (buff, " %2.1fC ",tmperature);
								//clear_buff (0);
								clear_buff (1);
								Print_Font (1,buff);
								
								delete_rain_down_new (2);
								clear_buff (0);
								scroll_center_in 	(2);
								delete_center_out 	(2);
								mode = CLOCK;
								Read_Clock (4);
								Print_Font (1,buff);
								shift_left (2);
								break;
				//-------------------------------------//
				//-------------------------------------//
				//-------------------------------------//
				default	:
								break;
								
			}
			flag = 0;					
		
		}
		
		
		
		if (interrupt)
		{
			printf ("%c",getByte);
			interrupt = 0;
		}
		_delay_us (10);		//Jan 15,2012 
							//jnut comment just to delay few 
		
	}

	while (1);
}
