#include <stdarg.h>
#include <string.h>
#include <avr/pgmspace.h>

#define SERIAL_OUT  //Output should go to serial-interface

#ifdef SERIAL_OUT
 #include "serial.h" 
#endif


#define SCRATCH 12	//32Bits go up to 4GB + 1 Byte for \0

//Spare some program space by making a comment of all not used format flag lines
#define USE_LONG 		// %lx, %Lu and so on, else only 16 bit integer is allowed
//#define USE_OCTAL		// %o, %O Octal output. Who needs this ?
#define USE_STRING      // %s, %S Strings as parameters
#define USE_CHAR		// %c, %C Chars as parameters
//#define USE_INTEGER	// %i, %I Remove this format flag. %d, %D does the same
#define USE_HEX			// %x, %X Hexadezimal output
#define USE_UPPERHEX	// %x, %X outputs A,B,C... else a,b,c...
#ifndef USE_HEX
 #undef USE_UPPERHEX    // ;)
#endif
#define USE_UPPER	// uncommenting this removes %C,%D,%I,%O,%S,%U,%X and %L..
                        // only lowercase format flags are used
#define PADDING         //SPACE and ZERO padding

//Making a comment of all #defines spares more then 360 Bytes.

//-------------------------------------------------------------------------//
//Outputs a single char. You can redirect output
//here to whatever you want.
void myputchar(unsigned char c)
{
#ifdef SERIAL_OUT
 if(c == '\n') //send CR and LF if \n
  {
   ser_putc(0x0D); //CR
   ser_putc(0x0A); //LF
  }
 else ser_putc(c);
#endif

}
//-------------------------------------------------------------------------//
//Outputs a text via myputchar().
void _puts_P(char const *s)
{
	uint8_t c;
		while((c=pgm_read_byte(s++))) myputchar(c);
}
//-------------------------------------------------------------------------//
void _printf_P(char const *format, ...)
{
	char 			scratch[SCRATCH];
	char 			*ptr;
	uint8_t 		format_flag;
	unsigned short 	base;
	
	uint8_t 		issigned = 0;
	va_list ap;

#ifdef USE_LONG
	uint8_t 	islong	= 0;
	uint32_t	u_val	= 0;
	long 		s_val   = 0;
#else
	uint16_t 	u_val=0;
	int 		s_val=0;
#endif

	uint8_t 	fill;
	uint8_t  	width;

	va_start (ap, format);
	for (;;)
	{
		while ((format_flag = pgm_read_byte(format++)) != '%')
		{      // Until '%' or '\0' 
			if (!format_flag){va_end (ap); 
					return;
		}
		myputchar(format_flag);
	}

		issigned=0; //default unsigned
    base = 10;

    format_flag = pgm_read_byte(format++); //get char after '%'

#ifdef PADDING
    width=0; //no formatting
    fill=0;  //no formatting
	if (format_flag=='0' || format_flag==' ') //SPACE or ZERO padding  ?
    {
		fill=format_flag;
		format_flag = pgm_read_byte(format++); //get char after padding char
		if(format_flag>='0' && format_flag<='9')
		{
			width = format_flag-'0';
			format_flag = pgm_read_byte(format++); //get char after width char
		}
	}
#endif

#ifdef USE_LONG
    islong=0; //default int value
#ifdef USE_UPPER
    if(format_flag=='l' || format_flag=='L') //Long value 
#else
    if(format_flag=='l') //Long value 
#endif
    {
		islong=1;
		format_flag = pgm_read_byte(format++); //get char after 'l' or 'L'
    }
#endif
    switch (format_flag)
    {
#ifdef USE_CHAR
		case 'c':
#ifdef USE_UPPER
		case 'C':
#endif
			format_flag = va_arg(ap,int);
			// no break -> run into default
#endif

			default:
					myputchar(format_flag);
			continue;

#ifdef USE_STRING
#ifdef USE_UPPER
		case 'S':
#endif
		case 's':
			ptr = va_arg(ap,char *);
			while(*ptr) { myputchar(*ptr); ptr++; }
			continue;
#endif

#ifdef USE_OCTAL
		case 'o':
#ifdef USE_UPPER
		case 'O':
#endif
		base = 8;
		myputchar('0');
		goto CONVERSION_LOOP;
#endif

#ifdef USE_INTEGER //don't use %i, is same as %d
		case 'i':
#ifdef USE_UPPER
		case 'I':
#endif
#endif
		case 'd':
#ifdef USE_UPPER
		case 'D':
#endif
      issigned=1;
      // no break -> run into next case
		case 'u':
#ifdef USE_UPPER
		case 'U':
#endif

//don't insert some case below this if USE_HEX is undefined !
//or put       goto CONVERSION_LOOP;  before next case.
#ifdef USE_HEX
				goto CONVERSION_LOOP;
		case 'x':
#ifdef USE_UPPER
		case 'X':
#endif
      base = 16;
#endif

CONVERSION_LOOP:

		if(issigned) //Signed types
		{
#ifdef USE_LONG
			if(islong) 	{ s_val = va_arg(ap,long); }
			else 		{ s_val = va_arg(ap,int); 	}
#else
			s_val = va_arg(ap,int);
#endif

			if(s_val < 0) //Value negativ ?
			{
				s_val = - s_val; //Make it positiv
				myputchar('-');    //Output sign
			}
			u_val = (uint32_t)s_val;
		}
		else //Unsigned types
		{
#ifdef USE_LONG
			if(islong) 	{ u_val = va_arg(ap,uint32_t); }
			else 		{ u_val = va_arg(ap,uint16_t); }
#else
			u_val = va_arg(ap,uint16_t);
#endif
       }
    
		ptr = scratch + SCRATCH;
		*--ptr = 0;
		do
		{
				char ch = u_val % base + '0';
#ifdef USE_HEX
				if (ch > '9')
				{
					ch += 'a' - '9' - 1;
#ifdef USE_UPPERHEX
					ch -= 0x20;
#endif
				}
#endif          
			*--ptr = ch;
			u_val /= base;
#ifdef PADDING
			if(width) width--; //calculate number of padding chars
#endif
		} while (u_val);

#ifdef PADDING
		while(width--) *--ptr = fill; //insert padding chars		      
#endif

		while(*ptr) { myputchar(*ptr); ptr++; }
		}
	}
}
