#include 	<avr/io.h>
#include 	<avr/interrupt.h>

#include 	<stdarg.h>
#include 	<string.h>
#include 	<avr/pgmspace.h>

#include 	"mydefs.h"
#include 	"serial.h"

#define RBUFLEN 16 

volatile 	uint8_t 	rbuf[RBUFLEN];
volatile 	uint8_t 	rcnt, 
						rpos, 
						recbuf;
volatile 	uint8_t 	busy;
extern 		uint8_t 	interrupt, getByte;

//---------------------------------------------------------------------------------//	
SIGNAL(SIG_UART0_RECEIVE)         
{
	getByte	= UART0_RECEIVE_REGISTER; 	
	interrupt = 1;	
}
//---------------------------------------------------------------------------------//	
SIGNAL(SIG_UART0_TRANSMIT)       
{
	DISABLE_UART0_TRANSMIT_INT; 
	busy=0; 
}
//-------------------------------------------------------------------------//
uint8_t ser_getc (void)
{
	uint8_t c;

	while(!rcnt) {};  

	DISABLE_UART0_TRANSMIT_INT; 
	rcnt--;
	
	c = rbuf [rpos++];
		if (rpos >= RBUFLEN)  	rpos = 0;
		ENABLE_UART0_RECEIVE_INT; 
	return c;
}
//-------------------------------------------------------------------------//
void ser_putc(uint8_t c)
{
	while(busy==1);	
		ENABLE_UART0_TRANSMIT_INT; 
		UART0_TRANSMIT_REGISTER=c;
	busy = 1;        
}
//-------------------------------------------------------------------------//
// Transmit string from RAM
void ser_puts(uint8_t *s)
{
	uint8_t c;

	while((c=*s++))
    {
		if(c == '\n') //CR und LF \n
		{
			ser_putc(0x0D); //CR
			ser_putc(0x0A); //LF
		}
		else 
			ser_putc(c);
    }
}

//-------------------------------------------------------------------------//
// Transmit string from FLASH
void _serputs_P(char const *s)
{
	uint8_t c;

	while((c=pgm_read_byte(s++)))
    {
		if(c == '\n') //CR und LF  \n
		{
			ser_putc(0x0D); //CR
			ser_putc(0x0A); //LF
		}
		else 
			ser_putc(c);
    }
}
//-------------------------------------------------------------------------//
void ser_puthex(uint8_t by)
{
	uint8_t	buff;

			buff=by>>4; 
			if(buff<10) 	buff+='0'; 
			else 			buff+=0x37;        
		
			ser_putc(buff);
			buff = by & 0x0f; 
		
			if(buff<10) 	buff+='0'; 
			else 			buff+=0x37;        
	ser_putc(buff);
}
//-------------------------------------------------------------------------//
void ser_init(void)
{
  
	rcnt = rpos = 0;  /// init buffers 
	busy = 0;

	TX0_DDR		|=	_BV(TX0_BIT);
	TX0_PORT 	|=	_BV(TX0_BIT);

	// enable RxD/TxD and ints  
	DISABLE_UART0_RECEIVE_INT;
	
	UART0_CONFIGURE1; // See serial.h
	UART0_CONFIGURE2; // See serial.h

	UART0_BAUD_REGISTER_HIGH	=(uint8_t)(UART_BAUD_SELECT>>8); 	// set baudrate
	UART0_BAUD_REGISTER_LOW		=(uint8_t)(UART_BAUD_SELECT); 		// set baudrate
	
	ENABLE_UART0_RECEIVE_INT;
}

